package com.googlecode.jpattern.gwt.client;

import java.util.Map;

import com.googlecode.jpattern.gwt.client.communication.AProxy;
import com.googlecode.jpattern.gwt.client.communication.ICallbackAction;
import com.googlecode.jpattern.gwt.client.communication.rest.DeleteProxy;
import com.googlecode.jpattern.gwt.client.communication.rest.GenericsObjectSerializer;
import com.googlecode.jpattern.gwt.client.communication.rest.GetProxy;
import com.googlecode.jpattern.gwt.client.communication.rest.PostProxy;
import com.googlecode.jpattern.gwt.client.communication.rest.PutProxy;
import com.googlecode.jpattern.gwt.shared.IWebResult;

/**
 * 
 * @author Francesco Cina'
 *
 * 06/mag/2011
 */
public class RestServerCallService implements IServerCallService {
	
	private final IBeanFactory beanFactory;

	public RestServerCallService(IBeanFactory beanFactory) {
		this.beanFactory = beanFactory;
	}

	@Override
	public <T extends IWebResult> AProxy<T> get(Class<T> resultClass,
			ICallbackAction<T> callbackAction, String url,
			Map<String, String> keyValuesMap) {
		return new GetProxy<T>(beanFactory, callbackAction, new GenericsObjectSerializer<T>(resultClass), url, keyValuesMap);
	}

	@Override
	public <T extends IWebResult> AProxy<T> delete(Class<T> resultClass,
			ICallbackAction<T> callbackAction, String url,
			Map<String, String> keyValuesMap) {
		return new DeleteProxy<T>(beanFactory, callbackAction, new GenericsObjectSerializer<T>(resultClass), url, keyValuesMap);
	}

	@Override
	public <T extends IWebResult, Z> AProxy<T> post(Class<T> resultClass, Class<Z> dataClass,
			ICallbackAction<T> callbackAction, String url, Z data) {
		return new PostProxy<T, Z>(beanFactory, callbackAction, new GenericsObjectSerializer<T>(resultClass), url, new GenericsObjectSerializer<Z>(dataClass), data);
	}

	@Override
	public <T extends IWebResult, Z> AProxy<T> put( Class<T> resultClass, Class<Z> dataClass,
			ICallbackAction<T> callbackAction, String url, Z data) {
		return new PutProxy<T, Z>(beanFactory, callbackAction, new GenericsObjectSerializer<T>(resultClass), url, new GenericsObjectSerializer<Z>(dataClass), data);
	}

}
