package com.googlecode.jpattern.gwt.client.communication;

import com.googlecode.jpattern.gwt.client.logger.ILogger;
import com.googlecode.jpattern.gwt.client.util.GenericWrapper;
import com.googlecode.jpattern.gwt.shared.IWebResult;
import com.googlecode.jpattern.shared.result.ErrorMessage;
import com.googlecode.jpattern.shared.result.IResult;

/**
 * 
 * @author Francesco Cina'
 *
 * 06/mag/2011
 */
public class ServerCallPostCommand<T extends IWebResult, Z> extends AServerCallCommand<T> {

	private final StringBuffer url;
	private final GenericWrapper<T> callResult;
	private final GenericWrapper<Z> dataInput;
	
	public ServerCallPostCommand(GenericWrapper<Z> dataInput, StringBuffer url, GenericWrapper<T> callResult) {
		this.url = url;
		this.callResult = callResult;
		this.dataInput = dataInput;
	}

	@Override
	protected void exec(IResult commandResult) {
		ILogger logger = getProvider().getLoggerService().getLogger(this.getClass());
		logger.debug("Start command execution");
		
		ServerCommandCallBack commandCallBack = new ServerCommandCallBack(commandResult, callResult);
		AProxy<T> proxy = getProvider().getServerCallService().post(callResult.getWrappedClass(), dataInput.getWrappedClass(), commandCallBack, url.toString(), dataInput.getValue());
		try {
			proxy.call();
		} catch (Exception exception) {
			commandResult.getErrorMessages().add(new ErrorMessage("Exception on Post Server call", exception.getMessage()));
			logger.error("Exception on Post Server call", exception);
		}
	}

}
