package com.googlecode.jpattern.gwt.client.communication.direct;

import java.util.Map;

import com.googlecode.jpattern.gwt.client.ApplicationProxy;
import com.googlecode.jpattern.gwt.client.communication.AProxy;
import com.googlecode.jpattern.gwt.client.communication.ICallbackAction;
import com.googlecode.jpattern.gwt.client.logger.ILogger;
import com.googlecode.jpattern.gwt.shared.IWebResult;

/**
 * 
 * @author Francesco Cina'
 *
 * @param <T>
 */
public class GetProxy<T extends IWebResult> extends AProxy<T> {

	private Map<String, String> keyValuesMap;
	private String url;
	private final IServerCallGetAction serverCallAction;
	private final Class<T> resultClass;

	public GetProxy(IServerCallGetAction serverCallAction, ICallbackAction<T> callbackAction, Class<T> resultClass, String url, Map<String, String> keyValuesMap) {
		super(callbackAction);
		this.serverCallAction = serverCallAction;
		this.resultClass = resultClass;
		this.url = url;
		this.keyValuesMap = keyValuesMap;
		
	}
	
	@Override
	protected void execute(ICallbackAction<T> callbackAction) throws Exception {
		ILogger logger = ApplicationProxy.getInstance().getApplicationProvider().getLoggerService().getLogger(this.getClass());

		logger.info("Begin execute - call URL " + url);
		try {
			callbackAction.onSuccess( serverCallAction.get(resultClass, url, keyValuesMap) );
		} catch (Exception e) {
			callbackAction.onError(e);
		}
		
	}
}
